@@ -115,6 +115,16 @@ Dp_Result * make_fast_generic_dp_pair_wise (Alignment *A, int*ns, int **l_s,Dp_M
 
 Constraint_list* free_dp_model  (Dp_Model *D);
 Dp_Result * free_dp_result (Dp_Result *D );
+
+typedef struct hseq* SeqHasch;
+
+typedef struct hseq
+{
+  SeqHasch hl[256];
+  int  n;
+  int  *l;
+} hseq;
+
 int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type);
 int ** evaluate_diagonals_for_two_sequences ( char *seq1, char *seq2,int maximise,Constraint_list *CL,int ktup);
 int ** evaluate_diagonals ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list,int ktup);
@@ -143,6 +153,11 @@ int precomputed_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
 int ktup_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
 int ktup_comparison ( char *seq1, char *seq2, int ktup);
 HaschT* hasch_sequence ( char *seq1, int ktup);
+
+SeqHasch * seq2hasch (int i,char *seq, int ktup, SeqHasch *H);
+Constraint_list * hasch2constraint_list (Sequence*S, Constraint_list *CL);
+SeqHasch *cleanhasch       (SeqHasch *H);
+int hasch2sim        (SeqHasch *H, int nseq);
 int cfasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
 int fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL);
 int make_fasta_gotoh_pair_wise_sw (Alignment *A,int*ns, int **l_s,Constraint_list *CL, int *diag);
@@ -158,8 +173,17 @@ int test_pair_wise (Alignment *A, int *ns, int **l_s, Constraint_list *CL);
 
 int glocal2_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL);
 int gotoh_pair_wise_lgp_sticky ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+int linked_pair_wise ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+int clinked_pair_wise ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+void free_proba_pair_wise();
+
 int subop1_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
 int subop2_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
+int proba_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
+int viterbi_pair_wise ( Alignment *A, int *ns, int **ls, Constraint_list *CL);
+int biphasic_pair_wise ( Alignment *A, int *ns, int **l_s, Constraint_list *CL);
+
+  
 
   
       
@@ -238,6 +262,8 @@ int * analyse_sequence ( Constraint_list *CL);
 /*                                                                          */	
 /****************************************************************************/
 Alignment * sorted_aln (Alignment *A, Constraint_list *CL);
+Alignment * sorted_aln_seq (int seq, Alignment *A, Constraint_list *CL);
+Alignment * full_sorted_aln (Alignment *A, Constraint_list *CL);
 
 /******************************************************************/
 /*                   MAIN DRIVER                                  */
@@ -279,19 +305,21 @@ Constraint_list *align_pdb_pair   (char *seq_in, char *dp_mode,char *evaluate_mo
 Constraint_list * align_pdb_pair_2 (char *seq, Constraint_list *CL);
 
 Constraint_list * pdb_pair  ( TC_method*M,char *seq, Constraint_list *CL);
+Constraint_list * pdbid_pair  ( TC_method*M,char *seq, Constraint_list *CL);
 Constraint_list * profile_pair  ( TC_method*M,char *seq, Constraint_list *CL);
 Constraint_list * thread_pair  ( TC_method*M,char *seq, Constraint_list *CL);
 Constraint_list * thread_pair2 ( TC_method *M,int s1, int s2, Constraint_list *CL);
 Constraint_list * sap_pair (char *seq, char *weight, Constraint_list *CL);
 Constraint_list * lsqman_pair (char *seq, Constraint_list *CL);
-
+Constraint_list * rna_pair (TC_method *M , char *in_seq, Constraint_list *CL);
 
 /******************************************************************/
 /*                   GENERIC PAIRWISE METHODS                     */
 /*                                                                */
 /*                                                                */
 /******************************************************************/
-
+Constraint_list *best_pair4prot (Job_TC *job);
+Constraint_list *best_pair4rna (Job_TC *job);
 Alignment * fast_pair      (Job_TC *job);
 			    
 void toggle_case_in_align_two_sequences(int value);
@@ -338,6 +366,7 @@ NT_node* tree_realn ( NT_node LT, NT_node RT, Alignment*A, int nseq, Constraint_
 
 int split_condition (int nseq, int score, Constraint_list *CL);
 
+int profile_pair_wise (Alignment *A, int n1, int *l1, int n2, int *l2, Constraint_list *CL);
 int pair_wise (Alignment *A, int*ns, int **l_s,Constraint_list *CL );
 
 int empty_pair_wise ( Alignment *A, int *ns, int **l_s, Constraint_list *CL, int glocal);
@@ -348,6 +377,7 @@ Pwfunc get_pair_wise_function (Pwfunc func, char *dp_mode, int *glocal);
 
 
 char *build_consensus ( char *seq1, char *seq2, char *dp_mode);
+int fastal (int argv, char **arg);
 
 int domain_pair_wise (Alignment *A,int*ns, int **l_s,Constraint_list *CL );
 Alignment *domain_match_list2aln ( Alignment *A,int *ns,int **l_s,int **ml, int nseq, int len);
@@ -395,6 +425,7 @@ float sap2_neighborhood_match (Constraint_list *CL, int s1, int s2, int r1, int
 /*                                                                                           */
 /*********************************************************************************************/
 Alignment * analyse_pdb ( Alignment *A, Alignment *ST, char *filename);
+Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *filename);
 float **** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *pdb_param);
 
 float square_atom ( Atom *X);
@@ -465,8 +496,12 @@ double lgamma_r(double x, int *signgamp);
 /*         FUNCTIONS FOR EVALUATING THE CONSISTENCY BETWEEN ALN AND CL                       */
 /*                                                                                           */
 /*********************************************************************************************/
+Alignment * overlay_alignment_evaluation     ( Alignment *I, Alignment *O);
 Alignment * main_coffee_evaluate_output          ( Alignment *IN,Constraint_list *CL, const char *mode );
-
+int aln2ecl_raw_score (Alignment *A, Constraint_list *C);
+int sub_aln2ecl_raw_score (Alignment *A, Constraint_list *CL, int ns, int *ls);
+int sub_aln2sub_aln_raw_score ( Alignment *IN,Constraint_list *CL, const char *mode, int *ns, int **ls);
+int node2sub_aln_score(Alignment *A,Constraint_list *CL, char *mode, NT_node T);
 int sub_aln2sub_aln_score ( Alignment *IN,Constraint_list *CL, const char *mode, int *ns, int **ls);
 Alignment *  main_coffee_evaluate_output_sub_aln ( Alignment *IN,Constraint_list *CL, const char *mode, int *ns, int **ls);
 
@@ -508,6 +543,8 @@ int evaluate_diaa_matrix_score                     ( Constraint_list *CL, int s1
 int evaluate_monoaa_matrix_score                   ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int evaluate_matrix_score                          ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int evaluate_tm_matrix_score                          ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int evaluate_ssp_matrix_score                          ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int evaluate_curvature_score                          ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 
 int evaluate_combined_matrix_score                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int evaluate_physico_score                         ( Constraint_list *CL, int s1, int r1, int s2, int r2);
@@ -519,7 +556,13 @@ int residue_pair_extended_list4rna3                 ( Constraint_list *CL, int s
 int residue_pair_extended_list4rna4                 ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list4rna                  ( Constraint_list *CL, Constraint_list *R, int s1, int r1, int s2, int r2);
 
+int residue_pair_extended_list_raw                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+int residue_pair_extended_list_pc                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list                     ( Constraint_list *CL, int s1, int r1, int s2, int r2);
+
+
+
+
 int residue_pair_extended_list_g_coffee_quadruplet ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list_g_coffee            ( Constraint_list *CL, int s1, int r1, int s2, int r2);
 int residue_pair_extended_list_quadruplet          ( Constraint_list *CL, int s1, int r1, int s2, int r2);
@@ -560,6 +603,7 @@ int check_fast_mode    (Alignment *A, int ns1,int *list1,int ns2, int *list2, Co
 
 
 int slow_get_dp_cost            ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
+int slow_get_dp_cost_pc         ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 int slow_get_dp_cost_test       ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
 
 int sw_get_dp_cost              ( Alignment *A, int**pos1, int ns1, int*list1, int col1, int**pos2, int ns2, int*list2, int col2, Constraint_list *CL);
@@ -696,7 +740,7 @@ int seq_viterbi_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL);
 /*                                                                               */
 /*********************************************************************************/
 
-int viterbi_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL);
+int viterbi_pair_wise_OLD (Alignment *A,int*ns, int **ls,Constraint_list *CL);
 Alignment * viterbipath2aln (Alignment *A, int *ns,int **ls,int *tb, Hmm *H);
 double*** viterbi_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list *CL);
 int * viterbi2path (int l1,int l2, Hmm *H, double ***M);
@@ -768,7 +812,7 @@ Sequence *pavie_seq2noisy_seq (Sequence *S, int val,char *subst);
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
