@@ -75,7 +75,24 @@ int apdb ( int argc, char *argv[])
 	int nirmsd_graph;
 	int n_template_file;
 	char **template_file_list;
+	char *mode; 
+		int prot_min_sim;
+	int prot_max_sim;
+	int prot_min_cov;
+	int pdb_min_sim;
+	int pdb_max_sim;
+	int pdb_min_cov;
 	
+
+	
+	char *prot_blast_server;
+	char *pdb_blast_server;
+	
+	
+	char *pdb_db;
+	char *prot_db;
+
+
 	argv=standard_initialisation (argv, &argc);
 	
 /*PARAMETER PROTOTYPE:    READ PARAMETER FILE     */
@@ -528,7 +545,7 @@ int apdb ( int argc, char *argv[])
 			    /*Min_value*/ "any"         ,\
 			    /*Max Value*/ "any"          \
 		   );
-
+	     
 	        declare_name (local_mode);
 		get_cl_param(					\
 			    /*argc*/      argc          ,\
@@ -565,19 +582,252 @@ int apdb ( int argc, char *argv[])
 			    /*Min_value*/ "any"         ,\
 			    /*Max Value*/ "any"          \
 					  );
+		/*PARAMETER PROTOTYPE:        MODE */
+		declare_name (mode);
+		get_cl_param(				  \
+			    /*argc*/      argc          , \
+			    /*argv*/      argv          , \
+			    /*output*/    &le           ,\
+			    /*Name*/      "-mode"         ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "S"           ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1           ,\
+			    /*DOC*/       "Mode: irmsd, ",\
+			    /*Parameter*/ &mode    ,	\
+			    /*Def 1*/    "irmsd",\
+			    /*Def 2*/     "stdin"       ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+					  );
+
 	
-	
+		
+	get_cl_param(\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-prot_min_sim"        ,\
+			    /*Flag*/      &prot_min_sim        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Minimum similarity between a sequence and its PDB target" ,\
+			    /*Parameter*/ &prot_min_sim          ,\
+			    /*Def 1*/     "0"             ,\
+			    /*Def 2*/     "20"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+ set_int_variable ("prot_min_sim", prot_min_sim);
+
+get_cl_param(\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-prot_max_sim"        ,\
+			    /*Flag*/      &prot_max_sim        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Maximum similarity between a sequence and its BLAST relatives" ,\
+			    /*Parameter*/ &prot_max_sim          ,\
+			    /*Def 1*/     "90"             ,\
+			    /*Def 2*/     "100"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+ set_int_variable ("prot_max_sim", prot_max_sim);
+ 
+get_cl_param(\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-prot_min_cov"        ,\
+			    /*Flag*/      &prot_min_cov        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Minimum coverage of a sequence by its BLAST relatives" ,\
+			    /*Parameter*/ &prot_min_cov          ,\
+			    /*Def 1*/     "0"             ,\
+			    /*Def 2*/     "0"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+set_int_variable ("prot_min_cov", prot_min_cov);
+
+get_cl_param(\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-pdb_min_sim"        ,\
+			    /*Flag*/      &pdb_min_sim        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Minimum similarity between a sequence and its PDB target" ,\
+			    /*Parameter*/ &pdb_min_sim          ,\
+			    /*Def 1*/     "35"             ,\
+			    /*Def 2*/     "35"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+
+ set_int_variable ("pdb_min_sim", pdb_min_sim);
+ get_cl_param(							\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-pdb_max_sim"        ,\
+			    /*Flag*/      &pdb_max_sim        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Maximum similarity between a sequence and its PDB target" ,\
+			    /*Parameter*/ &pdb_max_sim          ,\
+			    /*Def 1*/     "100"             ,\
+			    /*Def 2*/     "0"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+ set_int_variable ("pdb_max_sim", pdb_max_sim);
+ get_cl_param(							\
+			    /*argc*/      argc             ,\
+			    /*argv*/      argv             ,\
+			    /*output*/    &le              ,\
+			    /*Name*/      "-pdb_min_cov"        ,\
+			    /*Flag*/      &pdb_min_cov        ,\
+			    /*TYPE*/      "D"              ,\
+			    /*OPTIONAL?*/ OPTIONAL         ,\
+			    /*MAX Nval*/  1                ,\
+			    /*DOC*/       "Minimum coverage of a sequence by its PDB target" ,\
+			    /*Parameter*/ &pdb_min_cov          ,\
+			    /*Def 1*/     "50"             ,\
+			    /*Def 2*/     "25"             ,\
+			    /*Min_value*/ "any"            ,\
+			    /*Max Value*/ "any"             \
+		   );
+set_int_variable ("pdb_min_cov", pdb_min_cov);
+ 
+
+ 
+declare_name (pdb_blast_server);
+ get_cl_param(\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-pdb_blast_server"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "W_F"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "ND"          ,\
+			    /*Parameter*/&pdb_blast_server       ,\
+			    /*Def 1*/    "EBI"      ,\
+			    /*Def 2*/    "default"      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+declare_name (prot_blast_server);
+ get_cl_param(\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-blast"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "W_F"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "ND"          ,\
+			    /*Parameter*/&prot_blast_server       ,\
+			    /*Def 1*/    ""      ,\
+			    /*Def 2*/    ""      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+ //make sure that -blast and -blast_server are both supported blast>blast_server 
+ if ( !prot_blast_server[0])
+   {
+     get_cl_param(						\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-blast_server"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "W_F"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "ND"          ,\
+			    /*Parameter*/&prot_blast_server       ,\
+			    /*Def 1*/    "EBI"      ,\
+			    /*Def 2*/    "default"      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+   }
+ set_string_variable ("blast_server", prot_blast_server);
+ 
+
+ 
+ declare_name (pdb_db);
+ get_cl_param(\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-pdb_db"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "W_F"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "Non Redundant PDB database"          ,\
+			    /*Parameter*/&pdb_db       ,\
+			    /*Def 1*/    "pdb"      ,\
+			    /*Def 2*/    "default"      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+ set_string_variable ("pdb_db", pdb_db);
+ 
+
+declare_name (prot_db);
+ get_cl_param(\
+			    /*argc*/      argc          ,\
+			    /*argv*/      argv          ,\
+			    /*output*/    &le           ,\
+			    /*Name*/      "-protein_db"    ,\
+			    /*Flag*/      &garbage      ,\
+			    /*TYPE*/      "W_F"         ,\
+			    /*OPTIONAL?*/ OPTIONAL      ,\
+			    /*MAX Nval*/  1             ,\
+			    /*DOC*/       "ND"          ,\
+ 			    /*Parameter*/&prot_db       ,\
+	 		    /*Def 1*/    "uniprot"      ,\
+			    /*Def 2*/    "default"      ,\
+			    /*Min_value*/ "any"         ,\
+			    /*Max Value*/ "any"          \
+		   );
+ // set the correct mode:
+ if ( strm (argv[0], "trmsd"))sprintf (mode, "trmsd");
+   
+ set_string_variable ("prot_db", prot_db);
+ 
 
 		if (argc==1){myexit (EXIT_SUCCESS);}
 		
 		if ( strm (outfile,"no"))n_out_aln_format=0;
-		
+	
 		get_cl_param( argc, argv,&le, NULL,NULL,NULL,0,0,NULL);	
 		prepare_cache (cache);	      
 		
+		
 		if (strm ( aln, ""))
 		  sprintf ( aln, "%s", argv[1]);
 		
+		if (!is_aln (aln))
+		  {
+		    printf_exit (EXIT_FAILURE, stderr, "\n\n---- ERROR: File %s must be a valid alignment [FATAL:%s-%s]\n\n",aln,argv[0], PROGRAM);
+		  }
 		
 		pdb_param=vcalloc ( 1, sizeof(Pdb_param));
 		
@@ -615,6 +865,7 @@ int apdb ( int argc, char *argv[])
 		S=read_seq_in_n_list (list_file, n_list, NULL, NULL);
 		
 		le=display_sequences_names ( S,le,0, 0);
+	
 		if ( n_template_file)
 		  {
 		    fprintf ( le, "\nLooking For Sequence Templates:\n");
@@ -625,20 +876,21 @@ int apdb ( int argc, char *argv[])
 			fprintf ( le, "]");
 		      }
 		  }
-		
-		
-		
+	
 		if ( !strm (run_name, "default"))
 		  {
-
 		    F=parse_fname(run_name);
 		    sprintf (F->name, "%s", F->full);
 		  }
-		else F=parse_fname (aln);
-		
+		else 
+		  {
+		    F=parse_fname (aln);
+		  }
+
 		for ( a=0; a< S->nseq; a++)
 		  {
 		    char *p;
+		    
 		    p=seq2T_value (S, a, "template_file", "_P_");
 		    
 		    if (p)sprintf (S->file[a], "%s",p);
@@ -647,32 +899,27 @@ int apdb ( int argc, char *argv[])
 		CL=declare_constraint_list ( S,NULL, NULL, 0,NULL, NULL);
 		CL->T=vcalloc (S->nseq,sizeof (Ca_trace*));
 		
+
 		for ( n_pdb=0,a=0; a<S->nseq; a++)
 		  {
-
 		    if ( !is_pdb_file ( S->file[a])){CL->T[a]=NULL;continue;}
-
 		    CL->T[a]=read_ca_trace    (S->file[a], "ATOM");
 		    CL->T[a]=trim_ca_trace    (CL->T[a], S->seq[a]);
 		    (CL->T[a])->pdb_param=pdb_param;
 		    n_pdb++;
 		  }
-		
+
 		A=declare_aln (S);
+	
+
 		A->residue_case=KEEP_CASE;
 		A=main_read_aln(aln, A);
-		
-		
 		EA=copy_aln (A, EA);
-		
 		A->CL=CL;
 		
-		
 		if ( strm (apdb_outfile, "default"))
 		  sprintf ( apdb_outfile, "%s.apdb_result", F->name);
 		
-		
-		
 		if ( n_pdb<2)
 		  {
 		    FILE *fp;
@@ -683,9 +930,13 @@ int apdb ( int argc, char *argv[])
 		    fprintf (fp, "[FATAL:%s]\n", PROGRAM);
 		    vfclose (fp);
 		  }
-		else
+		else if ( strm (mode, "irmsd"))
 		  {
-		    EA=analyse_pdb ( A, EA, apdb_outfile);
+		      EA=analyse_pdb ( A, EA, apdb_outfile);
+		  }
+		else if ( strm (mode, "msa2tree") || strm (mode, "trmsd"))
+		  {
+		    EA=msa2struc_dist ( A, EA,F->name);
 		  }
 		le=display_output_filename ( le, "APDB_RESULT", "APDB_RESULT_FORMAT_01", apdb_outfile, CHECK);
 		
@@ -1341,8 +1592,6 @@ float **** irmsdmin_apdb_filtration ( Alignment *A, float ****residues, Constrai
 	    list[n][1]=(int)100000*residues[s1][s2][pos[s1][col1]-1][4];
 	    sum2+=residues[s1][s2][pos[s1][col1]-1][4];
 	    n++;
-	    
-	    
 	  }
 
 	if (n==0)return residues;
@@ -1589,9 +1838,8 @@ Alignment * analyse_pdb ( Alignment *A, Alignment *ST, char *results)
 		      fprintf ( pairwise, "\n\tPAIRWISE EVALUATED: %6.2f %%    [%s Vs %s] ",  (pair_len==0)?-1:(pair_tot*100)/pair_len,A->name[s1], A->name[s2]);
 		      fprintf ( pairwise, "\n\tPAIRWISE APDB:      %6.2f %%    [%s Vs %s] ",  (pair_tot==0)?-1:(pair_m3*100)/pair_tot,A->name[s1], A->name[s2]);
 		      fprintf ( pairwise, "\n\tPAIRWISE iRMSD:     %6.2f Angs [%s Vs %s]",  (pair_tot==0)?-1:pair_m4/pair_tot,A->name[s1], A->name[s2]);
-		      fprintf ( pairwise, "\n\tPAIRWISE NiRMSD:    %6.2f Angs [%s Vs %s]", (pair_tot==0)?-1:(pair_m4*pair_len)/(pair_tot*pair_tot), A->name[s1], A->name[s2]);
-		      
-		      if (PP->print_rapdb)fprintf (pairwise, "\n\tRAPDB PAIRS PAIRWISE N_NONEMPTY_PAIRS %d N_MAXIMUM_PAIRS %d",(int) pair_tot, (int)pair_len);
+		      fprintf ( pairwise, "\n\tPAIRWISE NiRMSD:    %6.2f Angs [%s Vs %s] [%d pos]", (pair_tot==0)?-1:(pair_m4*pair_len)/(pair_tot*pair_tot), A->name[s1], A->name[s2], (int)pair_tot);
+		      fprintf ( pairwise, "\n\tRAPDB PAIRS PAIRWISE N_NONEMPTY_PAIRS %d N_MAXIMUM_PAIRS %d",(int) pair_tot, (int)pair_len);
 		      msa_m3+=pair_m3;
 		       msa_m4+=pair_m4;
 		       msa_tot+=pair_tot;
@@ -1870,6 +2118,279 @@ float **** analyse_pdb_residues ( Alignment *A, Constraint_list *CL, Pdb_param *
 
 
 
+Alignment * msa2struc_dist ( Alignment *A, Alignment *ST, char *results)
+     {
+
+       int **pos, c;
+       FILE *tl;
+	 int s1, s2, rs1, rs2;
+	 int col1, col2;
+	 float ****distances;
+	 float **dm;
+	 int **count;
+	 int   **dm_int;
+	 float min, max;
+	 
+	      /*Distances[Nseq][len_aln][4]
+                distances...[0]: Number of residues within the bubble
+                distances...[1]: Absolute difference of distance of residues within Bubble
+	        distances...[2]: Number of residues within the bubble with Delta dist < md_threshold
+		distances ..[3]: Sum of squared difference of distances
+		distances ..[4]: iRMSD
+	      */
+	 Pdb_param *pdb_param;
+	 Constraint_list *CL;
+	 int a, b, ncol;
+	 float d1, d2,delta;
+	 int wd1, wd2;
+	 int in_bubble=0;
+	 int real_res1_col1=0;
+	 int real_res1_col2;
+	 int real_res2_col1;
+	 int real_res2_col2;
+	 Pdb_param *PP;
+	 int print_rapdb;
+	 float nrapdb, rapdb;
+	 Alignment *BA=NULL;
+	 NT_node *T0,*T1,*T2,*PT, *POS;
+	 NT_node BT0, BT10,BT50, BT100,RBT;
+	 char **pair_pos_list;
+	 
+	 int ntree=0, ntree2;
+	 
+	 Alignment *B;
+	 char *pos_list;
+	 char *tot_pos_list;
+	 char *struc_tree10;
+	 char *struc_tree100;
+	 char *struc_tree50;
+	 char *struc_tree0;
+	 
+	 char *color_struc_tree;
+	 int **score;
+	 int proceed=1;
+	 
+
+	 
+	 declare_name(tot_pos_list);
+	 sprintf ( tot_pos_list, "%s.tot_pos_list", results);
+	 
+	 declare_name(pos_list);
+	 sprintf ( pos_list, "%s.pos_list", results);
+
+	 declare_name(struc_tree0);
+	 sprintf ( struc_tree0, "%s.struc_tree_full",results);
+
+	 declare_name(struc_tree10);
+	 sprintf ( struc_tree10, "%s.struc_tree10",results);
+
+	 declare_name(struc_tree100);
+	 sprintf ( struc_tree100, "%s.struc_tree100",results);
+
+	 declare_name(struc_tree50);
+	 sprintf ( struc_tree50, "%s.struc_tree50",results);	 
+	 
+	 declare_name(color_struc_tree);
+	 sprintf ( color_struc_tree, "%s.struc_tree.html", results);
+	 
+	 pair_pos_list=declare_char (A->len_aln*A->len_aln+1, 100);  
+	 T1=vcalloc (A->len_aln*A->len_aln+1, sizeof (NT_node));
+	 T2=vcalloc (A->len_aln+1, sizeof (NT_node));
+	 
+	 PT=vcalloc (A->len_aln*A->len_aln+1, sizeof (NT_node));
+	 POS=vcalloc (A->len_aln+1, sizeof (NT_node));
+	 
+	 CL=A->CL;
+
+	 //Check all sequences have a PDB structure
+	 for (a=0; a<A->nseq; a++)
+	   {
+	     if ( ! seq2P_template_file(A->S,a))
+	       {
+		 fprintf ( stderr, "\n--- ERROR: %s has no structural template. All sequence in the MSA must have a known structure [FATAL]\n", (A->name[a]));
+		 proceed=0;
+	       }
+	   }
+	 if (!proceed)
+	    printf_exit (EXIT_FAILURE, stderr, "\n\n---- ERROR: All provided sequences must have a valid PDB identifier [FATAL:tRMSD-%s]\n\n", PROGRAM);
+	 
+	 for ( s1=0; s1< (A->S)->nseq; s1++)
+	   if ( CL->T[s1]){PP=(CL->T[s1])->pdb_param;break;}
+	 	 
+	 for ( s1=0; s1< A->nseq; s1++)
+	   {
+	     rs1=A->order[s1][0];
+	     if (CL->T[rs1] &&  !(CL->T[rs1])->ca_dist)(CL->T[rs1])->ca_dist=measure_ca_distances(CL->T[rs1]);
+	   }	 
+	 pos=aln2pos_simple (A, A->nseq);
+	 dm=declare_float (A->nseq, A->nseq);
+	 dm_int=declare_int (A->nseq, A->nseq);
+	 count=declare_int (A->nseq, A->nseq);
+	 PP->maximum_distance=10000;
+	 
+	 tl=vfopen (tot_pos_list, "w");
+	 for (ncol=0,ntree=0, col1=0; col1< A->len_aln; col1++)
+	   {
+	     int tree, cont;
+	     output_completion (stderr, col1, A->len_aln,1, "Sample Columns");
+	     for (cont=1,s1=0; s1<A->nseq; s1++)if ( is_gap (A->seq_al[s1][col1]))cont=0;//Stop if gap in column 1
+	     
+	     if ( cont==0)continue;
+	     
+	     for (s1=0; s1<A->nseq; s1++)for ( s2=0; s2<A->nseq; s2++){count[s1][s2]=0;dm[s1][s2]=0;}
+	     
+	     for ( ntree2=0,col2=0; col2<A->len_aln; col2++)
+	       {
+		 for (s1=0; s1< A->nseq-1; s1++)
+		   {
+		     for ( s2=s1+1; s2<A->nseq; s2++)
+		       {
+			 rs1=A->order[s1][0];
+			 rs2=A->order[s2][0];
+			 cont=1;
+			 
+			 if ( s1==s2){dm[s1][s2]=0;continue;}
+			 else if (!(CL->T[rs1]) || !(CL->T[rs2])){cont=0;}
+			 else if ( islower (A->seq_al[s1][col1]) || islower ( A->seq_al[s2][col1])){cont=0;}
+			 else if ( pos[s1][col1]<=0 || pos[s2][col1]<=0 ){cont=0;}
+			 if (pos[s1][col2]<=0 || pos[s2][col2]<=0 ){cont=0;}//stop if Gap in Column 2
+			 else if ( FABS((pos[s1][col2]-pos[s1][col1]))<=PP->n_excluded_nb){cont=0;}
+			 else if ( FABS((pos[s2][col2]-pos[s2][col1]))<=PP->n_excluded_nb){cont=0;}
+			 else if ( islower (A->seq_al[s1][col2]) || islower ( A->seq_al[s2][col2])){cont=0;}
+			 if ( cont==0){continue;}
+			  
+			 
+			 real_res1_col1=pos[s1][col1]-1;
+			 real_res1_col2=pos[s1][col2]-1;
+			 
+			 real_res2_col1=pos[s2][col1]-1;
+			 real_res2_col2=pos[s2][col2]-1;
+			 
+			 d1=(CL->T[rs1])->ca_dist[real_res1_col1][real_res1_col2];
+			 d2=(CL->T[rs2])->ca_dist[real_res2_col1][real_res2_col2];
+		
+			 if ( d1==UNDEFINED || d2 == UNDEFINED) continue;
+			 
+			 if ( strm ( PP->local_mode, "sphere"))
+			   {
+			     in_bubble= (d1<PP->maximum_distance && d2<PP->maximum_distance)?1:0;		   ;		   
+			   }
+			 else if ( strm ( PP->local_mode, "window"))
+			   {
+			     wd1=FABS((pos[s1][col2]-pos[s1][col1]));
+			     wd2=FABS((pos[s2][col2]-pos[s2][col1]));
+			     in_bubble= (wd1<PP->maximum_distance && wd2<PP->maximum_distance)?1:0;		   ;		   
+			   } 
+			 if (in_bubble)
+			   {
+			     delta=FABS((d1-d2));
+			     //delta=delta*delta;
+			     dm[s1][s2]=dm[s2][s1]+=delta;
+			     count[s1][s2]++;
+			     count[s2][s1]++;
+			   }
+		       }
+		   }
+	       }
+	   
+	     
+	     min=max=-1;
+	     for (tree=1,s1=0; s1<A->nseq-1; s1++)
+	       for (s2=s1+1; s2<A->nseq; s2++)
+		 {
+		   if ( count [s1][s2])dm[s1][s2]=dm[s2][s1]=dm[s1][s2]/(float)count[s1][s2];
+		   else 
+		     {
+		       tree=0;
+		     }
+		   if (s1==0 && s2==1)min=max=dm[s1][s2];
+		   min=MIN(dm[s1][s2], min);
+		   max=MAX(dm[s1][s2], max);
+		 }
+	     if (!tree || min==-1)continue;
+	     for (s1=0; s1<A->nseq-1; s1++)
+	       for (s2=s1+1; s2<A->nseq; s2++)
+		 {
+		   dm_int[s1][s2]=dm_int[s2][s1]=((dm[s1][s2])/(max))*100;
+		 }
+	     POS[col1]=T1[ntree]=compute_std_tree_2 ( A, dm_int, "_TMODE_upgma");
+	     fprintf (tl, "\n>Tree_%d Column\n", col1+1);
+	     print_tree (T1[ntree], "newick", tl);
+	     ntree++;
+	   }
+	 
+	 vfclose (tl);
+	 if (!ntree)
+	   {
+	      fprintf ( stderr, "\nERROR: No suitable pair of column supporting a tree [FATAL]\n", (A->name[a]));
+	      exit (EXIT_SUCCESS);
+	   }
+	 
+	 score=treelist2avg_treecmp (T1, NULL);
+	 
+	 display_output_filename( stdout,"TreeList","newick",tot_pos_list, CHECK);
+	 
+	 if ((BT10=treelist2filtered_bootstrap (T1, NULL,score, 0.1)))
+	   {
+	     vfclose (print_tree (BT10,"newick", vfopen (struc_tree10, "w")));
+	     display_output_filename( stdout,"Tree","newick",struc_tree10, CHECK);
+	   }
+	 
+	 if ((BT50=treelist2filtered_bootstrap (T1, NULL, score,0.5)))
+	   {
+	     vfclose (print_tree (BT50,"newick", vfopen (struc_tree50, "w")));
+	     display_output_filename( stdout,"Tree","newick",struc_tree50, CHECK);
+	   }
+	 
+	 if ((BT100=treelist2filtered_bootstrap (T1, NULL,score, 1.0)))
+	   {
+	     vfclose (print_tree (BT100,"newick", vfopen (struc_tree100, "w")));
+	     display_output_filename( stdout,"Tree","newick",struc_tree100, CHECK);
+	   }
+	 
+	 
+	 RBT=BT100;
+	 if (RBT)
+	   {
+	     B=copy_aln (A, NULL);
+	     for (a=0; a<A->len_aln; a++)
+	       {
+		 int score;
+		 Tree_sim *S;
+		 
+		 if (POS[a])
+		   {
+		     S=tree_cmp (POS[a], RBT);
+		     score=S->uw/10;
+		     vfree (S);
+		   }
+		 else
+		   {
+		     score=NO_COLOR_RESIDUE;
+		   }
+		 
+		 for (b=0; b<B->nseq; b++)
+		   {
+		     if ( is_gap (B->seq_al[b][a]) || score == NO_COLOR_RESIDUE)
+		       {
+			 B->seq_al[b][a]=NO_COLOR_RESIDUE;
+		       }
+		     else
+		       {
+			 B->seq_al[b][a]=S->uw/10;
+		       }
+		   }
+	       }
+	     
+	     output_format_aln ("score_html", A,B,color_struc_tree);
+	     display_output_filename( stdout,"Colored MSA","score_html",color_struc_tree, CHECK);
+	     free_aln (BA);
+	   }
+	 free_int (pos, -1);
+	 exit (EXIT_SUCCESS);
+	 return NULL;
+     }
+
 float square_atom ( Atom *X)
 {
   
@@ -2086,7 +2607,6 @@ char * rna_struc2rna_lib ( char *seq_name, char *seq, char *name)
 
   
   st=nussinov (seq, 2);
-  HERE (" %s", st);
   if ( name==NULL)name=vtmpnam(NULL);
   fp=vfopen ( name, "w");
   fprintf (fp, "! TC_LIB_FORMAT_01\n");
@@ -2127,7 +2647,7 @@ int display_rna_ss ( int n, char *seq, char *st, FILE *fp)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
