@@ -358,7 +358,7 @@ int * seq_viterbi2path (char *s, Hmm *H, double **M)
 /*                                                                               */
 /*********************************************************************************/
 Hmm* define_mnm_model(Constraint_list *CL);
-int viterbi_pair_wise (Alignment *A,int*ns, int **ls,Constraint_list *CL)
+int viterbi_pair_wise_OLD (Alignment *A,int*ns, int **ls,Constraint_list *CL)
 {
   int l1, l2, a;
   double ***M;
@@ -2514,7 +2514,7 @@ MatState* viterbiL_hmm (Alignment *A,int *ns, int **ls, Hmm *H, Constraint_list
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
