@@ -3,6 +3,7 @@
 #include <ctype.h>
 #include <string.h>
 #include <ctype.h>
+#include <stdlib.h>
 #include "io_lib_header.h"
 #include "util_lib_header.h"
 #include "define_header.h"
@@ -47,7 +48,7 @@ Sequence * cw_read_sequences ( char *seq_name)
 char *    get_string_type   (char *S)
         {
 	int a, l;
-	int protein=0,  dna=0, tot=0;
+	int protein=0,  dna=0,rna=0, tot=0;
 	char *type;
 	static char *ltype;
 	static int warning;
@@ -70,12 +71,14 @@ char *    get_string_type   (char *S)
 			{
 			protein+=( is_aa (S[a]) && !is_dna(S[a]));
 			dna+=    ( is_dna(S[a]));
+			rna+=    ( is_rna(S[a]));
 			tot++;
 			}
 		}
 
 	protein=(protein*100)/tot;
 	dna=(dna*100)/tot;
+	rna=(rna*100)/tot;
 	
 	if ( l<20 && warning==0)
 	  {
@@ -93,6 +96,7 @@ char *    get_string_type   (char *S)
 	      sprintf ( type, "PROTEIN");
 	    
 	  }
+	else if ( dna>98 && rna>10)sprintf ( type, "RNA");
 	else if ( dna>98)sprintf ( type, "DNA");
 	else sprintf ( type, "PROTEIN");
 	
@@ -159,6 +163,9 @@ char *set_mode (int mode, char *val)
   return type;
 }
 /************************************************************/
+
+  
+	
 char *unset_seq_type ()
 {
   return set_seq_type (UNSET, NULL);
@@ -246,7 +253,7 @@ void get_sequence (char *seq_file,int *NSEQ, char ***SEQ, char ***SN, int **sl,
 
 int ** get_matrix   ( char *name, char *format)
        {
-       
+
        if ( strm ( "blast", format))return read_blast_matrix ( name);
        else if ( strm ( "clustalw", format))return read_matrice(name);
        else
@@ -668,6 +675,7 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
     Color *box_c;
     Color *white;
     Alignment *S;
+
     
     S=copy_aln (B, NULL);
     
@@ -681,10 +689,13 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	if (i==-1)continue;
 	sprintf (buf2, "%s", Sin->seq_al[i]);ungap(buf2);
 	sprintf (buf3, "%s", S->seq_al[a]);ungap(buf3);
+	
 	if ( strlen (buf2) !=strlen(buf3))
 	  {
-	    fprintf ( stderr, "\nERROR: Incompatible cache\n");
-	    fprintf ( stderr, "\n%s\n%s", buf2, buf3); exit (EXIT_FAILURE);
+	    
+	    fprintf ( stderr, "\nERROR: Incompatible cache ON sEQ: %s\n", S->name[a]);
+	    fprintf ( stderr, "\n%s\n%s", buf2, buf3); 
+	    fprintf ( stderr, "\n\n%s\n%s", Sin->seq_al[i],S->seq_al[a]); exit (EXIT_FAILURE);
 	  }
 	
 	for (n=0,b=0;b<B->len_aln; b++)
@@ -730,6 +741,9 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
    fps->line-=max_len;
    fps->line=fps->line-fps->line%3;
    
+  
+   
+   
    for (a=0; a<B->len_aln; a+=fps->line)
 	   {	   
 	   
@@ -740,35 +754,35 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 	  
 	   for (b=0; b<=B->nseq; b++)
 	     {
-	     sprintf (buf,"%-*.*s ",max_len+2, max_len,S->name[b]);
-	     fps=print_format_string ( buf,white, ink, fps);
-	     if(B->output_res_num)
+	       sprintf (buf,"%-*.*s ",max_len+2, max_len,(b==B->nseq)?"":S->name[b]);
+	       fps=print_format_string ( buf,white, ink, fps);
+	       if(B->output_res_num)
 		 {
-		 sprintf (buf, " %4d ", n_residues[b]+1);
-		 fps=print_format_string ( buf,white, ink, fps);
+		   sprintf (buf, " %4d ", n_residues[b]+1);
+		   fps=print_format_string ( buf,white, ink, fps);
 		 }
-	     		 
-	     for (fps->in_seq=1,c=a;c<a+fps->line && c<B->len_aln;c++)
-		{
-		if (b==B->nseq)
-		   {
-		   n_residues[b]++;
-		   get_rgb_values_format (DEFAULT_COLOR,box_c);
-		   s=analyse_aln_column ( B, c);
-		   }
-		else
-		  {
-		    n_residues[b]+=!is_gap(B->seq_al[b][c]);
-		    s=B->seq_al[b][c]; 
-		    if (!is_gap(s) && S->seq_al[b][c]!=NO_COLOR_RESIDUE )
-		      {
-			get_rgb_values_format ( S->seq_al[b][c], box_c);			    			
-		      }
-		    else
-		      {
-			get_rgb_values_format (GAP_COLOR, box_c);		   	
-		      }
-		  }
+	       
+	       for (fps->in_seq=1,c=a;c<a+fps->line && c<B->len_aln;c++)
+		 {
+		   if (b==B->nseq)
+		     {
+		       n_residues[b]++;
+		       get_rgb_values_format (DEFAULT_COLOR,box_c);
+		       s=analyse_aln_column ( B, c);
+		     }
+		   else
+		     {
+		       n_residues[b]+=!is_gap(B->seq_al[b][c]);
+		       s=B->seq_al[b][c]; 
+		       if (!is_gap(s) && S->seq_al[b][c]!=NO_COLOR_RESIDUE )
+			 {
+			   get_rgb_values_format ( S->seq_al[b][c], box_c);			    			
+			 }
+		       else
+			 {
+			   get_rgb_values_format (GAP_COLOR, box_c);		   	
+			 }
+		     }
 		fps=print_format_char ( s,box_c, ink,fps);
 		}
 	      fps->in_seq=0;
@@ -908,7 +922,14 @@ FILE_format* (*vfclose_format)         ( FILE_format *))
 		{
 		if (b==S->nseq)
 		   {
-		   if (S->score_seq)get_rgb_values_format ( S->seq_al[b][c],box_c);
+		     
+		   if (S->score_seq)
+		     {
+		       int s;
+		       s=S->seq_al[b][c];
+		       if ( s>='0' && s<='9')s-='0';
+		       get_rgb_values_format (s,box_c);
+		     }
 		   else get_rgb_values_format (DEFAULT_COLOR,box_c);
 		   n_residues[b]++;
 		   s=analyse_aln_column ( B, c);
@@ -1112,6 +1133,95 @@ FILE_format* vfclose_ps ( FILE_format *fps)
 /*                       HTML FUNCTIONS                               */
 /*                                                                           */
 /*****************************************************************************/
+
+/*****************************************************************************/
+//JM_ADD
+/*****************************************************************************/
+void output_hit_matrix(char *fileName, float **ffpHitScoreMatrix, int nl)
+{
+	int i, j;
+     	FILE *fp;
+
+     	fp=vfopen(fileName, "w");
+ 	for(i = 0; i < nl; i++)
+	{
+		for(j = 0; j < i; j++)
+			fprintf(fp, "%6.2f ", ffpHitScoreMatrix[j][i-j]);
+		for(j = i; j < nl; j++)
+			fprintf(fp, "%6.2f ", ffpHitScoreMatrix[i][j-i]);
+		fprintf(fp, "\n");
+	}
+	vfclose(fp);
+}
+
+int output_hit_color_html (Alignment *B, float **ffPScoreTable, int nl, char *name)
+{
+   output_hit_color_format (B, ffPScoreTable, nl, name, vfopen_html,print_html_string,print_html_char,get_rgb_values_html, vfclose_html);
+   return 1;
+}
+
+int output_hit_color_format (Alignment *B, float **ffPScoreTable, int nl, char *name, \
+FILE_format *(*vfopen_format)          ( char *),\
+FILE_format *(*print_format_string)    ( char * ,Color *, Color *, FILE_format*),\
+FILE_format *(*print_format_char)      ( int    ,Color *, Color *, FILE_format*),\
+void         (*get_rgb_values_format)  ( int    ,Color *),\
+FILE_format* (*vfclose_format)         ( FILE_format *))
+{
+	int a, b;
+	int max_name_len=15;
+	int max_len=0;
+    
+	static char *buf;
+	static FILE_format *fps;
+	Color *ink;
+	Color *box_c;
+	Color *white;
+
+	box_c=vcalloc ( 1, sizeof (Color));    
+	get_rgb_values_format (DEFAULT_COLOR, (white=vcalloc ( 1, sizeof (Color))));	
+	get_rgb_values_format (INK_COLOR,     (ink  =vcalloc ( 1, sizeof (Color))));
+
+	if ( max_len==0)
+	{
+		for ( a=0; a< B->nseq; a++)
+		{
+			if ( strlen (B->name[a])>max_len)
+				max_len= strlen ( (B->name[a]));
+		}
+	}
+	if ( max_len>max_name_len)max_len=max_name_len;
+
+     	if ( buf==NULL)
+ 		buf=vcalloc (10000, sizeof (int));
+	int iEmptyChr = 32; //SPACE ASCIICODE
+	int iColorValue;
+	fps=vfopen_format(name);
+   	for (a=0; a < nl; a++)
+	{	   
+	     	sprintf (buf,"%*d ", max_len+2, a);
+	     	fps=print_format_string ( buf,white, ink, fps);
+		for(b = 0; b < a; b++)
+		{
+		      iColorValue = (int)((ffPScoreTable[b][a-b]*9)/100);
+		      get_rgb_values_format (iColorValue, box_c);
+		      fps=print_format_char (iEmptyChr,box_c, ink,fps);
+		}
+		for(b = a; b < nl; b++)
+		{
+		      iColorValue = (int)((ffPScoreTable[a][b-a]*9)/100);
+		      get_rgb_values_format (iColorValue, box_c);
+		      fps=print_format_char (iEmptyChr,box_c, ink,fps);
+		}
+	      fps=print_format_char ('\n', white, ink, fps);
+	}
+	vfclose_format(fps);
+	vfree(buf);
+	vfree(box_c);
+	return 1;
+}
+
+/*****************************************************************************/
+
 int       output_color_html     ( Alignment *B,Alignment *S, char *name)
       {
       output_color_format (B, S, name, vfopen_html,print_html_string,print_html_char,get_rgb_values_html, vfclose_html);
@@ -1264,7 +1374,7 @@ FILE_format* vfclose_html ( FILE_format *fhtml)
 int       output_color_ascii     ( Alignment *B,Alignment *S, char *name)
       {
 	output_color_format (B, S, name, vfopen_ascii,print_ascii_string,print_ascii_char,get_rgb_values_ascii, vfclose_ascii);
-      return 1;
+	return 1;
       }
 int       output_reliability_ascii     ( Alignment *B,Alignment *S, char *name)
       {
@@ -1355,7 +1465,7 @@ int       output_seq_reliability_ascii     ( Alignment *B,Alignment *S, char *na
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
