@@ -1,10 +1,10 @@
 int run_default (char *pg,char *filename);
-int check_configuration4program();
+//int check_configuration4program();
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
