@@ -12,6 +12,13 @@
 #include "io_lib_header.h"
 #include "util_lib_header.h"
 #include "define_header.h"
+#include "dp_lib_header.h"
+
+void free_pair_wise()
+{
+  //Free static allocated memory
+  free_proba_pair_wise();
+}
 
 /************************************************************************/
 /*                                                                      */
@@ -24,7 +31,7 @@ Constraint_list * declare_constraint_list_simple ( Sequence *S)
   return declare_constraint_list (S, NULL, NULL, 0, NULL, NULL);
 }
 
-Constraint_list * declare_constraint_list ( Sequence *S, char *name, int **L, int ne,FILE *fp, int **M)
+Constraint_list * declare_constraint_list ( Sequence *S, char *name, int *L, int ne,FILE *fp, int **M)
     {
     Constraint_list *CL;
 
@@ -62,7 +69,7 @@ Constraint_list *free_constraint_list4lib_computation (Constraint_list *CL)
 {
   if (!CL)return NULL;
   
-  free_int (CL->L, -1);
+  vfree (CL->L);
   free_int (CL->M, -1);
   
   vfree (CL);
@@ -163,6 +170,7 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
       NCL->el_size=CL->el_size;
 
     /*Normalisation information*/
+      NCL->filter_lib=CL->filter_lib;
       NCL->normalise=CL->normalise;
       NCL->overweight=CL->overweight;
       NCL->max_ext_value=CL->max_ext_value;
@@ -201,6 +209,7 @@ Constraint_list *copy_constraint_list (Constraint_list *CL, int mode)
       sprintf (NCL->tree_mode, "%s", CL->tree_mode);
       NCL->tree_aln=(mode==HARD_COPY)?copy_aln (CL->tree_aln, NULL):CL->tree_aln;
     /*Functions used for dynamic programming and Evaluation*/ 
+      NCL->no_overaln=CL->no_overaln;
       NCL->profile_mode=CL->profile_mode;
       sprintf ( NCL->profile_comparison, "%s",CL->profile_comparison);
       NCL->get_dp_cost=CL->get_dp_cost;
@@ -344,7 +353,7 @@ Sequence *free_constraint_list (Constraint_list *CL)
 
 
     
-    if ( CL->L)free_int (CL->L, -1);
+    if ( CL->L)vfree (CL->L);
     if ( CL->M)free_int (CL->M, -1);
     if ( CL->fp)vfclose (CL->fp);
     if ( CL->matrices_list)free_char(CL->matrices_list,-1);
@@ -537,6 +546,7 @@ Sequence * declare_sequence ( int min, int max, int nseq)
     LS->max_nseq=nseq;
     LS->type=vcalloc(30, sizeof (char));
     LS->T=declare_arrayN(2, sizeof (Template), nseq, 1);
+    LS->dc=declare_int (nseq, 2);
     return LS;
     }
 Sequence * realloc_sequence   (Sequence *OUT, int new_nseq, int max_len)
@@ -558,7 +568,7 @@ Sequence * realloc_sequence   (Sequence *OUT, int new_nseq, int max_len)
       OUT->len     =vrealloc     ( OUT->len,     (new_nseq+1)*sizeof (int));
       
       OUT->T=(Template**)realloc_arrayN (2, (void **)OUT->T,sizeof (Template), new_nseq, 1);
-      
+      OUT->dc=(int **)realloc_arrayN (2, (void **)OUT->dc,sizeof (int), new_nseq, 2);
       OUT->max_nseq=new_nseq;
       return OUT;
     }
@@ -566,33 +576,39 @@ Sequence * realloc_sequence   (Sequence *OUT, int new_nseq, int max_len)
 Sequence * duplicate_sequence (Sequence *S )
     {
     Sequence *LS;
-    int a;
+    int a, b;
     
 
     if (S==NULL)return S;
     LS=declare_sequence (S->min_len, S->max_len, S->nseq);
-        
-    for ( a=0; a<S->nseq; a++)
+    for (b=0, a=0; a<S->nseq; a++)
         {
-	sprintf ( LS->file[a], "%s", S->file[a]);	
-	sprintf ( LS->seq_comment[a], "%s", S->seq_comment[a]);
-	sprintf ( LS->aln_comment[a], "%s", S->aln_comment[a]);
-	
-	sprintf ( LS->seq[a], "%s", S->seq[a]);
-	sprintf ( LS->name[a], "%s", S->name[a]);
-	LS->len[a]=S->len[a];
-	
-	LS->T[a][0]=S->T[a][0];
+	  if (S->seq && S->seq[a])
+	    {
+	      
+	      sprintf ( LS->file[b], "%s", S->file[a]);	
+	      if ( S->seq_comment && S->seq_comment[a])sprintf ( LS->seq_comment[b], "%s", S->seq_comment[a]);
+	      if ( S->aln_comment && S->aln_comment[a])sprintf ( LS->aln_comment[b], "%s", S->aln_comment[a]);
+	      if ( S->seq && S->seq[a])sprintf  ( LS->seq[b], "%s", S->seq[a]);
+	      if ( S->name&& S->name[a])sprintf ( LS->name[b], "%s", S->name[a]);
+	      LS->dc[b][0]=S->dc[a][0];
+	      LS->dc[b][1]=S->dc[a][1];
+	      LS->len[b]=S->len[a];
+	      LS->T[b][0]=S->T[a][0];
+	      b++;
+	      
+	    }
 	}
-
+ 
     LS->max_len=S->max_len;
     LS->min_len=S->min_len;
-    LS->nseq=S->nseq;
+    LS->nseq=b;
 
     if ( S->W)LS->W=duplicate_weights (S->W);
     sprintf ( LS->type, "%s", S->type);
     sprintf ( LS->template_file, "%s", S->template_file);
     LS->max_nseq=S->nseq;
+    
     return LS;
     }
 
@@ -608,7 +624,7 @@ void free_sequence ( Sequence *LS, int nseq)
 	free_char ( LS->aln_comment, -1);
 	free_char ( LS->seq, -1);
 	free_char ( LS->name,-1);
-	
+	free_int  (LS->dc, -1);
 	free_arrayN((void*)LS->T, 2);
 	vfree (LS->type);
 	vfree (LS->len);
@@ -639,19 +655,23 @@ Weights* declare_weights ( int nseq)
 Weights* duplicate_weights (Weights *W)
      {
        Weights *NW;
-       int a, b;
+       int a, b, c;
 
        NW=declare_weights (W->nseq);
        sprintf ( NW->comments, "%s", W->comments);
        sprintf ( NW->mode, "%s", W->mode);
-       for (a=0; a< W->nseq; a++)
+       for (a=0, c=0; a< W->nseq; a++)
 	 {
-	   sprintf ( NW->seq_name[a], "%s", W->seq_name[a]);
-	   NW->SEQ_W[a]=W->SEQ_W[a];
-	   for(b=0; b< W->nseq; b++)
+	   if ( W->seq_name[a])
 	     {
-	       	NW->PW_SD[a][b]=W->PW_SD[a][b];
-		NW->PW_ID[a][b]=W->PW_ID[a][b];
+	       sprintf ( NW->seq_name[c], "%s", W->seq_name[a]);
+	       NW->SEQ_W[c]=W->SEQ_W[a];
+	       for(b=0; b< W->nseq; b++)
+		 {
+		   NW->PW_SD[c][b]=W->PW_SD[a][b];
+		   NW->PW_ID[c][b]=W->PW_ID[a][b];
+		 }
+	       c++;
 	     }
 	 }
        return NW;
@@ -684,10 +704,11 @@ Alignment* copy_aln ( Alignment *A, Alignment *B)
 
 	  
 	  if ( A==NULL){free_aln(B); return NULL;}	    
-	  free_aln (B);
-	 
 	  
-	  if ( A->S && A->nseq>(A->S)->nseq)
+	  
+	  if (B)
+	    B=realloc_alignment2 (B, A->nseq, A->len_aln);
+	  else if ( A->S && A->nseq>(A->S)->nseq)
 	    {
 	      B=declare_aln2(A->nseq, MAX((A->S)->max_len+1, A->len_aln+1));
 	      B->S=A->S;
@@ -1041,7 +1062,7 @@ long aln_stack (Alignment *A, int mode)
       return 0;
     }
 }
-		 
+
 Sequence* free_aln ( Alignment *LA){ return free_Alignment(LA);}
 Sequence* free_Alignment ( Alignment *LA)
 	{
@@ -1784,7 +1805,7 @@ Fname *free_fname ( Fname *F)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
