@@ -17,6 +17,10 @@ int *code_seq (char *seq, char *type);
 int * makepointtable( int *pointt, int *n, int ktup );
 
 static int tsize;
+
+/**
+* calculates the number of common tuples
+*/
 int commonsextet( short *table, int *pointt )
 {
 	int value = 0;
@@ -28,8 +32,8 @@ int commonsextet( short *table, int *pointt )
 		
 	if( !memo )
 	{
-		memo = vcalloc( tsize, sizeof( short ) );
-		ct = vcalloc( tsize, sizeof( int ) );
+		memo = vcalloc( tsize+1, sizeof( short ) );
+		ct = vcalloc( tsize+1, sizeof( int ) );
 	}
 
 	cp = ct;
@@ -52,6 +56,9 @@ int commonsextet( short *table, int *pointt )
 	return( value );
 }
 
+/**
+*	calculates how many of each tupble exist 
+*/
 void makecompositiontable( short *table, int *pointt )
 {
 	int point;
@@ -61,6 +68,7 @@ void makecompositiontable( short *table, int *pointt )
 	    table[point]++;
 	  }
 }
+
 int *code_seq (char *seq, char *type)
 {
   static int *code;
@@ -71,7 +79,7 @@ int *code_seq (char *seq, char *type)
   if (!aa)
     {
       char **gl;
-      if ( strm (type, "DNA"))
+      if ( strm (type, "DNA") || strm (type, "RNA"))
 	{
 	  gl=declare_char (4,5);
 	  sprintf ( gl[ng++], "Aa");
@@ -115,7 +123,8 @@ int *code_seq (char *seq, char *type)
   code[a]=END_ARRAY;
   return code;
 }
-    
+
+
 int * makepointtable( int *pointt, int *n, int ktup )
 {
   int point, a, ng;
@@ -157,31 +166,27 @@ int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type)
 {
   //Adapted from MAFFT 5: fast ktup
   int **pointt,*code=NULL, **pscore;
-  int i, l, j;
-  
+  int i, l, j, minl;
   double **mtx, score0;
   
-
-  for (l=0,i=0;i<nseq; i++)
+  
+  if (!seq || nseq==0)return NULL;
+  for (minl=strlen(seq[0]),l=0,i=0;i<nseq; i++)
     {
-      l=MAX(l, strlen(seq[i]));
+      int len;
+      len=strlen (seq[i]);
+      minl=MIN(minl, len);
+      l=MAX(l,len);
     }
-
-  pointt=declare_int (nseq, l);
+  ktup=MIN(minl, ktup);
+  pointt=declare_int (nseq, l+1);
   mtx=declare_double (nseq, nseq);
   pscore=declare_int ( nseq, nseq);
   
-  //Make sure ktup is not longer than the shortest sequence
-  for (i=0; i<nseq; i++)
-    {
-      ungap(seq [i]);
-      ktup=MIN(ktup, strlen (seq[i]));
-    }
-  
   for( i=0; i<nseq; i++ ) 
-    {
+  {
       makepointtable( pointt[i], code=code_seq (seq[i], type),ktup);
-    }
+  }
   tsize=(int)pow(code[-1], ktup);
  
   for ( i=0; i<nseq; i++)
@@ -189,13 +194,12 @@ int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type)
       short *table1;
       table1=vcalloc ( tsize,sizeof (short));
       makecompositiontable( table1, pointt[i]);
-      
       for (j=i; j<nseq; j++)
 	{
 	  mtx[i][j] = commonsextet( table1, pointt[j] );
 	}
+      vfree (table1);
     }
-
   for( i=0; i<nseq; i++ )
     {
       score0 = mtx[i][i];
@@ -209,7 +213,7 @@ int ** ktup_dist_mat ( char **seq, int nseq, int ktup, char *type)
       }
     return pscore;
 }
- 
+
 
 int ** evaluate_diagonals_with_ktup_1 ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list, int ktup);
 int ** evaluate_diagonals_with_ktup_2 ( Alignment *A, int *ns, int **l_s, Constraint_list *CL,int maximise,int n_groups, char **group_list, int ktup);
@@ -242,7 +246,7 @@ int ** evaluate_diagonals_for_two_sequences ( char *seq1, char *seq2,int maximis
 		   CL->evaluate_residue_pair= evaluate_matrix_score;
 	      else if (  strm(type, "PROTEIN"))
 		   CL->evaluate_residue_pair=evaluate_matrix_score;
-	      else if (  strm (type, "DNA"))
+	      else if (  strm (type, "DNA") || strm (type, "RNA"))
 		   CL->evaluate_residue_pair= evaluate_matrix_score;
 	      vfree(type);
 	    }
@@ -1668,11 +1672,148 @@ int ** evaluate_diagonals_with_ktup_1 ( Alignment *A, int *ns, int **l_s, Constr
     free_int (pos, -1);
     return diag;
     }
+/////////////////////////////////////////////////////////////////
+
+Constraint_list * hasch2constraint_list (Sequence*S, Constraint_list *CL)
+{
+  int a,b,c, n;
+  SeqHasch h,*H=NULL;
+  int *entry;
+  int ktup=2;
+  
+  
+  entry=vcalloc ( CL->entry_len, sizeof (int));
+
+  for (a=0; a<S->nseq; a++)
+    {
+      H=seq2hasch (a, S->seq[a],ktup,H);
+    }
+ 
+  n=1;
+  while (H[n])
+    {
+      h=H[n];
+
+      for (a=0; a<h->n-2; a+=2)
+	{
+	  for (b=a+2; b<h->n; b+=2)
+	    {
+	      
+	      if (h->l[a]==h->l[b])continue;
+	      else
+		{
+		  for (c=0; c<ktup; c++)
+		    {
+		      entry[SEQ1]=h->l[a];
+		      entry[SEQ2]=h->l[b];
+		      entry[R1]=h->l[a+1]+c;
+		      entry[R2]=h->l[b+1]+c;
+		      entry[WE]=100;
+		      add_entry2list (entry,CL);
+		    }
+		}
+	    }
+	}
+      n++;
+    }
+
+  return CL;
+}
+SeqHasch *cleanhasch       (SeqHasch *H)
+{
+  int n=1;
+  SeqHasch *N;
+  N=vcalloc (2, sizeof (SeqHasch));
+  N[0]=H[0];
+  
+  while (H[n])
+    {
+      (H[n])->n=0;
+      vfree ((H[n])->l);
+      (H[n])->l=NULL;
+      n++;
+    }
+  vfree (H);
+  return N;
+}
+int hasch2sim        (SeqHasch *H, int nseq)
+{
+  int n=1;
+  
+  int a,cs, ps, ns;
+  int id=0, tot=0;
+  
+  while (H[n])
+    {
+      for (ps=-1,ns=0,a=0; a<(H[n])->n; a+=2)
+	{
+	  //HERE ("%d--[%d %d]",n, (H[n])->l[a], (H[n])->l[a+1]); 
+	  cs=(H[n])->l[a];
+	  if (cs!=ps)ns++;
+	  ps=cs;
+	}
+      n++;
+      if (ns==nseq)id++;
+      tot++;
+    }
+
+  return (id*MAXID)/tot;
+}
+SeqHasch * seq2hasch (int i,char *seq, int ktup, SeqHasch *H)
+{
+  int a,b,l, n=0;
+  SeqHasch h;
+
+  
+  if (!H)
+    {
+      H=vcalloc (2, sizeof (SeqHasch));
+      H[0]=vcalloc (1, sizeof (hseq));
+      n=1;
+    }
+  else
+    {
+      n=0;
+      while (H[++n]);
+    }
+  
+  l=strlen (seq);
+  for (a=0; a<l-ktup; a++)
+    {
+      h=H[0];
+      for (b=a; b<a+ktup; b++)
+	{
+	  char r;
+	  r=seq[b];
+	  if (!h->hl[r])  h->hl[r]=vcalloc (1, sizeof (hseq));
+	  h=h->hl[r];
+	}
+      if (!h->l)
+	{
+
+	  h->n=2;
+	  h->l=vcalloc (2, sizeof (int));
+	  H=vrealloc (H,(n+2)*sizeof (SeqHasch));
+	  H[n]=h;
+	  n++;
+	}
+      else 
+	{
+	  h->n+=2;
+	  h->l=vrealloc (h->l, (h->n)*sizeof (int));
+	}
+
+      h->l[h->n-2]=i;
+      h->l[h->n-1]=a;
+    }
+  return H;
+}
+
 /*********************************COPYRIGHT NOTICE**********************************/
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
