@@ -40,7 +40,7 @@ Alignment * graph_aln (Alignment *A, Constraint_list *iCL, Sequence *S)
   CL=duplicate_constraint_list (iCL);
   for ( a=0; a<CL->ne; a++)
     {
-      CL->L[a][WE]=CL->evaluate_residue_pair ( iCL, CL->L[a][SEQ1],CL->L[a][R1] ,CL->L[a][SEQ2],CL->L[a][R2]);
+      CL->L[a*CL->entry_len+WE]=CL->evaluate_residue_pair ( iCL, CL->L[a*CL->entry_len+SEQ1],CL->L[a+CL->entry_len+R1] ,CL->L[a*CL->entry_len+SEQ2],CL->L[a*CL->entry_len+R2]);
      }
    CL=sort_constraint_list_on_n_fields(CL, 0, CL->ne,WE,1);
    G=aln2graph (A);
@@ -543,7 +543,7 @@ int check_link (CL_node ***G, int s1, int r1, int s2, int r2)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
