@@ -19,6 +19,11 @@ static double 	*left_branch, *right_branch;
 static int 	*tkill;
 
 
+/*!
+ *	\file util_make_tree.c
+ *	\brief Source code tree algorithms
+ */
+
 NT_node ** seq2cw_tree ( Sequence *S, char *tree)
 {
   Alignment *A;
@@ -37,7 +42,21 @@ NT_node ** seq2cw_tree ( Sequence *S, char *tree)
   return read_tree (tree, &tot_node, S->nseq, S->name);
 }
 
+NT_node ** make_upgma_tree (  Alignment *A,int **distances,int gop, int gep, char **out_seq, char **out_seq_name, int out_nseq, char *tree_file, char *tree_mode)
+  {
 
+    if ( distances==NULL && A==NULL)
+      {
+	fprintf ( stderr, "\nError: make_nj_tree, must provide an alignment or a distance matrix [FATAL:%s]", PROGRAM);
+	myexit (EXIT_FAILURE);
+      }
+    else if ( distances==NULL)
+      {
+
+	distances=get_dist_aln_array (A, "idmat");
+      }
+    return int_dist2upgma_tree (distances,A, A->nseq,tree_file);
+  }
 NT_node ** make_nj_tree (  Alignment *A,int **distances,int gop, int gep, char **out_seq, char **out_seq_name, int out_nseq, char *tree_file, char *tree_mode)
   {
 
@@ -63,7 +82,9 @@ NT_node ** int_dist2nj_tree (int **distances, char **out_seq_name, int out_nseq,
 	  for ( a=0; a< out_nseq; a++)
 	    for ( b=0; b< out_nseq; b++)
 	      d[a][b]=distances[a][b];
+	  
 	  T=dist2nj_tree ( d, out_seq_name, out_nseq, tree_file);
+	  
 	  free_double (d, -1);
 	  return T;
 	}
@@ -86,7 +107,7 @@ NT_node ** dist2nj_tree (double **distances, char **out_seq_name, int out_nseq,
 	int a, b;
 	double **d_dist;
 	int tot_node=0;
-
+	NT_node **T;
 
 	if ( !tree_file)tree_file=vtmpnam(NULL);
 	d_dist=declare_double( out_nseq+1, out_nseq+1);
@@ -101,8 +122,9 @@ NT_node ** dist2nj_tree (double **distances, char **out_seq_name, int out_nseq,
 
 	guide_tree ( tree_file, d_dist, out_seq_name, out_nseq);
 	free_double (d_dist,-1);
-	
-	return read_tree (tree_file,&tot_node,out_nseq,  out_seq_name);	
+	T=read_tree (tree_file,&tot_node,out_nseq,  out_seq_name);	
+
+	return T;
 	}
 
 
@@ -337,7 +359,7 @@ void slow_nj_tree(char **tree_description)
 	    
 
 	  }
-	/**end main cycle**/
+	/*end main cycle**/
 	
 /******************************Last Cycle (3 Seqs. left)********************/
 
@@ -1002,7 +1024,7 @@ void fast_nj_tree(char **tree_description)
 			tmat[minj][j] = tmat[j][minj] = join[j] = 0.0;
 
 
-/****/	}						/**end main cycle**/
+/****/	}						/*end main cycle**/
 
 /******************************Last Cycle (3 Seqs. left)********************/
 
@@ -1061,55 +1083,46 @@ void fast_nj_tree(char **tree_description)
 //                              UPGMA_aln
 //////////////////////////////////////////////////////////////////////////////
 
-Alignment * upgma_merge_aln_rows  (Alignment***mat,int *used, int *n, int N, Constraint_list *CL);
-Alignment * upgma_merge_aln_pairs (Alignment *L, Alignment *R,Constraint_list *CL);
+Alignment * upgma_merge_aln_rows (Alignment *A, int *ns, int **ls,int N, int**mat,int *used, int *n,  Constraint_list *CL);
+int upgma_pair_wise (Alignment *A, int *ls0, int ns0, int *ls2, int ns2, Constraint_list *CL);
+
 
 Alignment * upgma_tree_aln  ( Alignment*A, int nseq, Constraint_list *CL)
 {
-  int a, b,c,n, *used;
-  Alignment *NA;
-  static Alignment ***mat;
+  int a, b,n, *used;
+  static int **mat;
+  int **ls;
+  int *ns;
+  nseq=(CL->S)->nseq;
+  mat=declare_int (nseq, nseq);
+  ls=declare_int  (nseq,nseq);
+  ns=vcalloc (nseq,sizeof (int));
   
-  if (!mat)
-    {
-      Alignment *aln;
-
-      mat=declare_arrayN (2, sizeof ( Alignment *), A->nseq, A->nseq);
-      int **ls;
-      int *ns;
-      ls=declare_int (2, 2);
-      ns=vcalloc (2,sizeof (int));
-      ns[0]=ns[1]=1;
-      for (a=0; a<A->nseq; a++)
-	for (b=a; b<A->nseq; b++)
-	  {
-	  
-	    mat[a][b]=mat[b][a]=aln=seq2aln ((CL->S),NULL, RM_GAP);
-	    ls[0][0]=a;
-	    ls[1][0]=b;
-	    for (c=0; c<aln->nseq; c++)if (c!=a && c!=b)aln->seq_al[c][0]='\0'; 
-	    if (a!=b)
-	      {
-		pair_wise(aln, ns, ls, CL);
-	    	aln->score_aln=aln->score=sub_aln2sub_aln_score (aln, CL, CL->evaluate_mode,ns, ls);
-	      }
-	  }
-      free_int (ls, -1);
-      vfree (ns);
-    }
-  used=vcalloc (A->nseq, sizeof (int));
-  n=A->nseq;
+  for (a=0; a<nseq-1; a++)
+    for (b=a+1; b<nseq; b++)
+      {
+	ns[a]=ns[b]=1;
+	ls[a][0]=a;
+	ls[b][0]=b;
+	mat[a][b]=mat[b][a]=upgma_pair_wise(A,ls[a],ns[a],ls[b],ns[b],CL);
+	HERE ("%d %d", a, b, mat[a][b]);
+      }
+  
+  used=vcalloc (nseq, sizeof (int));
+  n=nseq;
   while (n>1)
     {
-      NA=upgma_merge_aln_rows (mat,used, &n, A->nseq, CL);
+      upgma_merge_aln_rows (A,ns, ls,nseq, mat, used, &n,CL);
     }    
-  print_aln (NA);
+  print_aln (A);
   HERE ("finished");
- 
-  return NA;
+  free_int ( mat, -1);
+  free_int (ls, -1);
+  vfree (ns);
+  return A;
 }
 
-Alignment * upgma_merge_aln_rows (Alignment***mat,int *used, int *n, int N, Constraint_list *CL)
+Alignment * upgma_merge_aln_rows (Alignment *A, int *ns, int **ls,int N, int**mat,int *used, int *n,  Constraint_list *CL)
 {
   
   int a, b, w, best_a, best_b, set;
@@ -1121,7 +1134,7 @@ Alignment * upgma_merge_aln_rows (Alignment***mat,int *used, int *n, int N, Cons
       for (b=a+1; b<N; b++)
 	{
 	  if (used[b])continue;
-	  w=(mat[a][b])->score_aln;
+	  w=mat[a][b];
 	  if ( !set || w>best_s)
 	    {
 	      best_s=w;
@@ -1132,68 +1145,82 @@ Alignment * upgma_merge_aln_rows (Alignment***mat,int *used, int *n, int N, Cons
 	}
     }
   used[best_b]=1;
-  mat[best_a][best_a]=copy_aln(mat[best_a][best_b], NULL);
   
-  for (a=0; a<N; a++)
+  //merge a and b
+  mat[best_a][best_b]=upgma_pair_wise (A, ls[best_a], ns[best_a], ls[best_b], ns[best_b], CL);
+  for (a=0; a<ns[best_b]; a++)ls[best_a][ns[best_a]++]=ls[best_b][a];
+  ns[best_b]=0;
+  
+  //update the a row
+  for (a=0; a< A->nseq; a++)
     {
-      if (used[a])continue;
-      mat[best_a][a]=mat[a][best_a]=upgma_merge_aln_pairs (mat[best_a][best_a],mat[a][a], CL);
+      if (a!=best_a && !used[a])
+	mat[best_a][a]=mat[a][best_a]=upgma_pair_wise (A, ls[best_a], ns[best_a], ls[a], ns[a], CL);
     }
+  
   HERE ("DONE");
   
   n[0]--;
-  return mat[best_a][best_a];
-}  
-Alignment * upgma_merge_aln_pairs (Alignment *L,Alignment *R, Constraint_list *CL)
-{
-  static int *ns;
-  static int **ls;
-  static Alignment *A;
-  int a;
-
-  if (!L || !R) return NULL;
-  if ( L==R) return L;
-  if (!ns)
-    {
-      ns=vcalloc ( 2, sizeof (int));
-      ls=declare_int (2, (CL->S)->nseq);
-    }
-  ns[0]=ns[1]=0;
-  A=seq2aln ((CL->S),NULL, RM_GAP);
-  A=copy_aln (L, A);
-
-  for (a=0; a<(CL->S)->nseq; a++)
-    {
-      A->seq_al[a][0]='\0';
-      if (L->seq_al[a][0])
-	{
-	  ls[0][ns[0]++]=a;
-	  sprintf (A->seq_al[a], "%s", L->seq_al[a]);
-	}
-      if (R->seq_al[a][0])
-	{
-	  ls[1][ns[1]++]=a;
-	  sprintf (A->seq_al[a], "%s", R->seq_al[a]);
-	}
-    }
-  pair_wise(A, ns, ls, CL);
-  A->score_aln=A->score=sub_aln2sub_aln_score (A, CL, CL->evaluate_mode,ns, ls);
   return A;
+}  
+int upgma_pair_wise (Alignment *A, int *ls0, int ns0, int *ls1, int ns1, Constraint_list *CL)
+{
+  static int **ls;
+  static int *ns;
+  static int *fl;
+  int a, b, n;
+  
+  if ( !ls )
+    {
+      ls=vcalloc (2, sizeof (int*));
+      ns=vcalloc (2, sizeof (int));
+      fl=vcalloc ((CL->S)->nseq, sizeof (int));
+    }
+  ls[0]=ls0;
+  ls[1]=ls1;
+  ns[0]=ns0; ns[1]=ns1;
+
+  fprintf ( stderr, "\n");
+  for (a=0; a<ns0; a++)
+    fprintf ( stderr, "%d", ls0[a]);
+  fprintf ( stderr, "\n");
+  for (a=0; a<ns1; a++)
+    fprintf ( stderr, "%d", ls1[a]);
+	      
+  ungap_sub_aln (A, ns[0], ls[0]);
+  ungap_sub_aln (A, ns[1], ls[1]);
+  HERE ("Align");
+  pair_wise (A, ns, ls, CL);
+  HERE ("Done");
+	
+  for (n=0,a=0;a<2; a++)
+    for (b=0; b<ns[a]; b++)
+      fl[n++]=ls[a][b];
+  
+  return sub_aln2ecl_raw_score (A,CL,n, fl);
 }
+  
 //////////////////////////////////////////////////////////////////////////////
 //
 //                              UPGMA
 ///////////////////////////////////////////////////////////////////////////////
 
-NT_node upgma_merge (int **mat, NT_node *NL, int *used, int *n, int N);
+
 NT_node ** int_dist2upgma_tree (int **mat, Alignment *A, int nseq, char *fname)
 {
   NT_node *NL, T;
   int a, n, *used;
+  int tot_node;
+  if (upgma_node_heap (NULL))
+    {
+      printf_exit ( EXIT_FAILURE,stderr, "\nERROR: non empty heap in upgma [FATAL]");
+    }
   NL=vcalloc (nseq, sizeof (NT_node));
+  
   for (a=0; a<A->nseq; a++)
     {
       NL[a]=new_declare_tree_node ();
+      upgma_node_heap (NL[a]);
       sprintf (NL[a]->name, "%s", A->name[a]);
       NL[a]->isseq=1;
       NL[a]->leaf=1;
@@ -1206,9 +1233,12 @@ NT_node ** int_dist2upgma_tree (int **mat, Alignment *A, int nseq, char *fname)
       T=upgma_merge (mat, NL,used, &n, A->nseq);
     }    
 
+  vfree (used);
   vfclose (print_tree (T, "newick", vfopen (fname, "w")));
-
-  return NULL;
+  upgma_node_heap (NULL);
+  vfree (NL);
+  
+  return read_tree (fname,&tot_node,A->nseq,  A->name);
 }
 NT_node upgma_merge (int **mat, NT_node *NL, int *used, int *n, int N)
 {
@@ -1216,7 +1246,8 @@ NT_node upgma_merge (int **mat, NT_node *NL, int *used, int *n, int N)
   int a, b, w, best_a, best_b, set;
   float best_s;
   P=new_declare_tree_node();
-    
+  upgma_node_heap (P);
+  
   for (set=0,a=0; a<N-1; a++)
     {
       if (used[a])continue;
@@ -1258,11 +1289,37 @@ NT_node upgma_merge (int **mat, NT_node *NL, int *used, int *n, int N)
 //
 //                              SPLIT UPGMA
 ///////////////////////////////////////////////////////////////////////////////
-
+int upgma_node_heap (NT_node X);
+int upgma_node_heap (NT_node X)
+{
+  static int n;
+  static NT_node *h;
+  if ( X==NULL)
+    {
+      int a,r;
+      if (n==0) return 0;
+      for (a=0; a<n; a++)
+	{
+	  free_tree_node (h[a]);
+	}
+      vfree (h);
+      h=NULL;
+      r=n;
+      n=0;
+      return r;
+    }
+  else
+    {
+      h=vrealloc (h, sizeof (NT_node)*(n+1));
+      h[n++]=X;
+    }
+  return n;
+}
 NT_node split2upgma_tree (Split **S, Alignment *A, int nseq, char *fname)
 {
   NT_node *NL, T;
   int a, n, *used;
+  
   NL=vcalloc (nseq, sizeof (NT_node));
   for (a=0; a<A->nseq; a++)
     {
@@ -1274,6 +1331,7 @@ NT_node split2upgma_tree (Split **S, Alignment *A, int nseq, char *fname)
       NL[a]->isseq=1;
       NL[a]->leaf=1;
       NL[a]->dist=1;
+      upgma_node_heap (NL[a]);
     }
   used=vcalloc ( A->nseq, sizeof (int));
   n=A->nseq;
@@ -1281,9 +1339,10 @@ NT_node split2upgma_tree (Split **S, Alignment *A, int nseq, char *fname)
     {
       T=split_upgma_merge (A,S, NL,used, &n, A->nseq);
     }    
+  vfree (used);
   fprintf ( stdout, "\n");
   vfclose (print_tree (T, "newick", vfopen (fname, "w")));
-
+  upgma_node_heap (NULL);
   return T;
 }
 NT_node split_upgma_merge (Alignment *A, Split **S, NT_node *NL, int *used, int *n, int N)
@@ -1304,6 +1363,7 @@ NT_node split_upgma_merge (Alignment *A, Split **S, NT_node *NL, int *used, int
     }
 
   P=new_declare_tree_node();
+  upgma_node_heap (P);
   P->lseq2=vcalloc (N, sizeof (int));
   for (set=0,a=0; a<N-1; a++)
     {
@@ -1385,7 +1445,7 @@ float get_split_dist ( Alignment *A, NT_node L, NT_node R, Split **S)
 /* Centro de Regulacio Genomica */
 /*and */
 /*Cedric Notredame */
-/*Fri Jun  6 17:35:06 WEST 2008. */
+/*Tue Oct 27 10:12:26 WEST 2009. */
 /*All rights reserved.*/
 /*This file is part of T-COFFEE.*/
 /**/
